int main()
{
    uint32_t conf_addr;
    uint8_t i;

    volatile uint8_t att_stat;

    // I2C communication
    uint8_t i2c_write_buf[3] = {0};
    uint8_t i2c_read_buf[3] = {0};
    tm_status_t tm_status = NO_CONNECTION;

    /* Enable this to delay the firmware execution under SWD connect. */
#ifdef BREAK_AT_MAIN
    uint8_t volatile x = 0;
    while(x==0);
#endif /* BREAK_AT_MAIN */

#if (CCG_SROM_CODE_ENABLE)
    srom_vars_init (CCG_IMG1_FW_METADATA_ADDR, CCG_IMG2_FW_METADATA_ADDR);
#endif /* (CCG_SROM_CODE_ENABLE) */

#if CCG_FIRMWARE_APP_ONLY
    CALL_IN_FUNCTION(sys_set_device_mode)(SYS_FW_MODE_FWIMAGE_1);
#else /* !CCG_FIRMWARE_APP_ONLY */
    if ((uint32_t)&base_version < CCG_FW1_CONFTABLE_MAX_ADDR)
    {
        CALL_IN_FUNCTION(sys_set_device_mode)(SYS_FW_MODE_FWIMAGE_1);
    }
    else
    {
        CALL_IN_FUNCTION(sys_set_device_mode)(SYS_FW_MODE_FWIMAGE_2);
    }
#endif /* CCG_FIRMWARE_APP_ONLY */

    /* Validate the signature and checksum of the configuration table. */
    conf_addr = (uint32_t)get_pd_config ();
    if ((boot_validate_configtable ((uint8_t *)conf_addr) != CCG_STAT_SUCCESS) ||
            (!app_validate_configtable_offsets()))
    {
#if CCG_FIRMWARE_APP_ONLY
        /* Can't do anything if config table is not valid. */
        while (1);
#else /* !CCG_FIRMWARE_APP_ONLY */
        /* Erase the firmware metadata so that this binary stops loading. */
        if (CALL_IN_FUNCTION(sys_get_device_mode)() == SYS_FW_MODE_FWIMAGE_1)
        {
            flash_row_clear (CCG_IMG1_METADATA_ROW_NUM);
        }
        else
        {
            flash_row_clear (CCG_IMG2_METADATA_ROW_NUM);
        }

        /* Now reset the device. */
        CySoftwareReset ();
#endif /* CCG_FIRMWARE_APP_ONLY */
    }

    /* Initialize PD block clocks. */
    system_init();

    /* Timer INIT has to be done first. */
    timer_init();

    /* Enable global interrupts */
    CyGlobalIntEnable;

#if RIDGE_SLAVE_ENABLE
    ridge_slave_init (RIDGE_SLAVE_SCB, 0);
#endif /* RIDGE_SLAVE_ENABLE */

#if CCG_HPI_ENABLE

#if CCG_FIRMWARE_APP_ONLY
    /* Configure HPI for no-boot operation. */
    hpi_set_no_boot_mode (true);
#endif /* CCG_FIRMWARE_APP_ONLY */

    /* Set the flash sizes and bootloader size limits. */
    hpi_set_flash_params (CCG_FLASH_SIZE, CCG_FLASH_ROW_SIZE, CCG_LAST_FLASH_ROW_NUM + 1, CCG_BOOT_LOADER_LAST_ROW);

    /* Initialize the HPI interface. */
    hpi_init (HPI_SCB_INDEX);

    /* Update HPI registers with mode and version information. */
    update_hpi_regs ();

#endif /* CCG_HPI_ENABLE */

    /* Initialize the instrumentation related data structures. */
    instrumentation_init();

    /* Configure the VBus detach detection parameters (same as VBus poll params). */
    pd_hal_set_vbus_detach_params (APP_VBUS_POLL_ADC_ID, APP_VBUS_POLL_ADC_INPUT);

#if (MUX_INIT_DELAY_MS != 0)
    /* Specify the MUX enable delay value to be used. */
    dpm_update_mux_enable_wait_period(MUX_INIT_DELAY_MS);
#endif /* (MUX_INIT_DELAY_MS != 0) */

#if CCG_UCSI_ENABLE
    /* Initialize the UCSI interface */
    ucsi_init ();
#endif /* CCG_UCSI_ENABLE */

    /* Perform application level initialization. */
    app_init();

    /* Initialize the Device Policy Manager for each PD port on the device. */
    for (i = TYPEC_PORT_0_IDX; i < NO_OF_TYPEC_PORTS; i++)
    {
        dpm_init (i, app_get_callback_ptr(i));
    }

#if CCG_HPI_ENABLE

    /* Send a reset complete event to the EC. */
    hpi_send_fw_ready_event ();

    /* Wait until EC ready event has been received or 100 ms has elapsed. */
    for (i = 0; i < 100; i ++)
    {
        hpi_task ();
        if (hpi_is_ec_ready ())
            break;

#if (CYDEV_BCLK__SYSCLK__MHZ == 48)
        /*
         * Using CyDelayCycles as CyDelay is leading to a 25% error.
         * The parameter used here assumes that CPU clock is running at 48 MHz.
         */
        CyDelayCycles (38400);
#else
        /*
         * Using CyDelayCycles as CyDelay is leading to a 25% error.
         * The parameter used here assumes that CPU clock is running at 24 MHz.
         */
        CyDelayCycles (19200);
#endif /* (CYDEV_BCLK__SYSCLK__MHZ == 48) */
    }

#endif /* CCG_HPI_ENABLE */

#if APP_FW_LED_ENABLE

    /* Configure the LED control GPIO as an output. */
    CALL_IN_FUNCTION(gpio_hsiom_set_config)(FW_LED_GPIO_PORT_PIN, HSIOM_MODE_GPIO, GPIO_DM_STRONG, false);
    /* Start a timer that will blink the FW ACTIVE LED, if required. */
    timer_start (0, LED_TIMER_ID, LED_TIMER_PERIOD, led_timer_cb);

#endif /* APP_FW_LED_ENABLE */

    /* Start any timers or tasks associated with application instrumentation. */
    instrumentation_start();

    for (i = TYPEC_PORT_0_IDX ; i < NO_OF_TYPEC_PORTS; i++)
    {
#if CCG_HPI_ENABLE
        /* Start the DPM for the port only if it is enabled at the HPI level. It is possible that the port
           got disabled before we got here.
         */
        if ((hpi_get_port_enable () & (1 << i)) != 0)
#endif /* CCG_HPI_ENABLE */
        {
            dpm_start (i);
        }
    }

    /* Init NCP81239 Power Delivery Controller */
    pd_ctrl_init();

    // Set VSEL so that only 5V is passed to VBUS
    set_5v_output();

    // Set Source Capability mask so that only 5V profile is enabled
    dpm_update_src_cap_mask(0, MASK_5V_ONLY);

    // Set MUX_SEL to a default option
    MUX_SEL_Write(1);

    // Start interrupt low
    I2C_INT_Write(0);

    CyGlobalIntEnable;

    // Initialize and start I2C
    I2C_I2CSlaveInitReadBuf(i2c_read_buf, 3);
    I2C_I2CSlaveInitWriteBuf(i2c_write_buf, 3);
    I2C_Start();
    set_i2c_addr();

    while(1)
    {
        /* Perform DPM, APP and HPI tasks. */
        for (i = TYPEC_PORT_0_IDX; i < NO_OF_TYPEC_PORTS; i++)
        {
            dpm_task (i);
            app_task (i);

#if CCG_HPI_ENABLE
            /* Handle any pending HPI commands. */
            hpi_task ();
#endif /* CCG_HPI_ENABLE */

#if CCG_UCSI_ENABLE
            /* Handle any pending UCSI commands. */
            ucsi_task ();
#endif /* CCG_UCSI_ENABLE */

            /* Perform tasks associated with instrumentation. */
            instrumentation_task();

            /////////       Handle Device Connection/Disconnection      /////////
            const dpm_status_t *dpm_stat = dpm_get_info (i);
            att_stat = dpm_stat->attach;// | dpm_stat->contract_exist;    // attach works for devices that don't respond to VDMs
            if (att_stat != false) {
                if (tm_status == NO_CONNECTION) {         // new connection
                    tm_status = CONNECTED;
                }
              } else if (att_stat == false) {
                if (tm_status != NO_CONNECTION) {// new disconnection
                    tm_status = NO_CONNECTION;
                }
            }
        }

#if SYS_DEEPSLEEP_ENABLE
        /* If possible, enter sleep mode for power saving. */
        system_sleep();
#endif /* SYS_DEEPSLEEP_ENABLE */
    }
}